import PQObject, types
from pTypes import *

class PQueryFactory:
  def __init__(self):
    " Initializes an empty store "
    self.store = dict()
    
  def pAnd(self, l1, l2):
    'list intersection'
    l = []
    for n in l1:
      if n in l2:
        l.append(n)
    return l

  def pOr(self, l1, l2):
    'list union'
    for n in l1:
      if n not in l2:
        l2.append(n)
    return l2
        
    
  def addStore(self, store):
    "  "
    store.pqf = self
    self.store[store.ns] = store

  def getNode(self, ptype, key, ns):
    """ Returns a PQObject
        guarantees the object exist in the db"""
    node = PQObject.PQObject(self, ptype, key, ns)
    if node.exist():
      return node
    return False
  '''  
  def getAlias(self, node):
    " returns all aliases of the node "
    l = []
    for s in self.store:
      for d in s.getAlias(node):
        exist = 0
        for r in l:
          if d.key == r.key and d.ns == r.ns:
            exist = 1
        if exist == 0:
          l.append(d) 
    return l
  '''
  '''
  def get(self, node, ptype):
    """ handles all kinds of relations """
    return self.store[node.pid.ns].get(node, ptype) 
  '''    
  def getAll(self, ptype):
    """  """
    l = []
    for s in self.store.itervalues():
      for d in s.getAll(ptype):
        if not d in l:
          l.append(d)
    return l

  def getAllAnnotated(self, ptype, arl):
    " returns all data that contributed directly "
    all = self.getAll(ptype)
    result = []
    for one in all:
      allok = True
      for (key, ttype, value) in arl:
        ok = False
        a = one.getAnnotation()
        if   ttype == 'eq':
          if a.has_key(key) and a[key] == value: ok = True
        elif ttype == 'neq':
          if not a.has_key(key) or (a.has_key(key) and a[key] != value): ok = True
        elif ttype == 'in':
          if a.has_key(key) and value in a[key]: ok = True
        elif ttype == 'nin':
          if not a.has_key(key) or (a.has_key(key) and value not in a[key]): ok = True
        elif ttype.lower() == 'weekday':
          for s in self.store.itervalues():
            if s.ns == one.pid.ns:
              if a.has_key(key) and s.dayOfWeek(a[key], value): ok = True
        if not ok:
          allok = False
      if allok:
        result.append(one)
    return result  
    
  def traverse(self, node1, node2, limit = 0, stop = []):
    if PQObject.PQObject != node2.__class__:
      return self.store[node1.pid.ns].downstream(node1, limit, stop)
    if PQObject.PQObject != node1.__class__:
      return self.store[node2.pid.ns].upstream(node2, limit, stop)
    return []


  ############ api passthroughs ########## 
  
  ## execution section ##
  def getDataFromOutPort(self, node):
    return self.store[node.pid.ns].getDataFromOutPort(node) 

  def getOutPortFromData(self, node):
    return self.store[node.pid.ns].getOutPortFromData(node) 

  def getDataFromInPort(self, node):
    return self.store[node.pid.ns].getDataFromInPort(node) 

  def getInPortFromData(self, node):
    return self.store[node.pid.ns].getInPortFromData(node) 

  def getExecutionFromInData(self, node):
    return self.store[node.pid.ns].getExecutionFromInData(node) 

  def getInDataFromExecution(self, node):
    return self.store[node.pid.ns].getInDataFromExecution(node) 

  def getExecutionFromOutData(self, node):
    return self.store[node.pid.ns].getExecutionFromOutData(node) 

  def getOutDataFromExecution(self, node):
    return self.store[node.pid.ns].getOutDataFromExecution(node) 

  def getInstanceFromExecution(self, node):
    return self.store[node.pid.ns].getInstanceFromExecution(node) 

  def getExecutionFromInstance(self, node):
    return self.store[node.pid.ns].getExecutionFromInstance(node) 

  def getRunFromExecution(self, node):
    return self.store[node.pid.ns].getRunFromExecution(node) 

  def getExecutionFromRun(self, node):
    return self.store[node.pid.ns].getExecutionFromRun(node) 

  def getRunFromWorkflow(self, node):
    return self.store[node.pid.ns].getRunFromWorkflow(node) 

  def getWorkflowFromRun(self, node):
    return self.store[node.pid.ns].getWorkflowFromRun(node) 

  def getInstanceFromWorkflow(self, node):
    return self.store[node.pid.ns].getInstanceFromWorkflow(node) 

  def getWorkflowFromInstance(self, node):
    return self.store[node.pid.ns].getWorkflowFromInstance(node) 

  def getParentDataItem(self, node):
    return self.store[node.pid.ns].getParentDataItem(node) 

  def getChildDataItem(self, node):
    return self.store[node.pid.ns].getChildDataItem(node) 

  def getParentExecution(self, node):
    return self.store[node.pid.ns].getParentExecution(node) 

  def getChildExecution(self, node):
    return self.store[node.pid.ns].getChildExecution(node) 

  ## workflow part ##

  def getModuleFromInstance(self, node):
    return self.store[node.pid.ns].getModuleFromInstance(node) 

  def getInstanceFromModule(self, node):
    return self.store[node.pid.ns].getInstanceFromModule(node) 

  def getStartConnectionFromInstance(self, node):
    return self.store[node.pid.ns].getStartConnectionFromInstance(node) 

  def getEndConnectionFromInstance(self, node):
    return self.store[node.pid.ns].getEndConnectionFromInstance(node) 
  
  def getStartInstanceFromConnection(self, node):
    return self.store[node.pid.ns].getStartInstanceFromConnection(node) 
  
  def getEndInstanceFromConnection(self, node):
    return self.store[node.pid.ns].getEndInstanceFromConnection(node) 
  
  def getConnectionFromWorkflow(self, node):
    return self.store[node.pid.ns].getConnectionFromWorkflow(node) 
  
  def getWorkflowFromConnection(self, node):
    return self.store[node.pid.ns].getWorkflowFromConnection(node) 
  
  def getParentWorkflow(self, node):
    return self.store[node.pid.ns].getParentWorkflow(node) 
  
  def getChildWorkflow(self, node):
    return self.store[node.pid.ns].getChildWorkflow(node) 
  
  def getConnectionFromInPort(self, node):
    return self.store[node.pid.ns].getConnectionFromInPort(node) 
  
  def getInPortFromConnection(self, node):
    return self.store[node.pid.ns].getInPortFromConnection(node) 
  
  def getConnectionFromOutPort(self, node):
    return self.store[node.pid.ns].getConnectionFromOutPort(node) 
  
  def getOutPortFromConnection(self, node):
    return self.store[node.pid.ns].getOutPortFromConnection(node) 
  
  ## module section ##
  def getOutPortFromModule(self, node):
    return self.store[node.pid.ns].getOutPortFromModule(node) 

  def getInPortFromModule(self, node):
    return self.store[node.pid.ns].getInPortFromModule(node) 

  def getModuleFromOutPort(self, node):
    return self.store[node.pid.ns].getModuleFromOutPort(node) 

  def getModuleFromInPort(self, node):
    return self.store[node.pid.ns].getModuleFromInPort(node)

  ## module section ##
  def getAnnotation(self, node):
    return self.store[node.pid.ns].getAnnotation(node) 

