from pTypes import * 
import PQObject
#querycount = 0
class PStore:
  """ Provenance store base class 
      Also implements aliases """
  def __init__(self):
    " Sets empty namespace value "
    self.ns = 'default'
    self.alias = dict() # data alias
    self.connList = [] # execution dependencies
  '''    
  ### Alias part ### (entity resolution) total chaos 
  # bug if add(a,b) and add(c,d) then add(a,c) will create duplicates
  def addAlias(self, ptype, pid1, pid2): # discrete set structure?
    " add an alias horrible code not used" 
    if not ptype in self.alias:
      self.alias[ptype] = []
    for ent in self.alias[ptype]: # for each registered entity
      if pid1.ns in ent: # if namespace is registered for entity
        if pid1.key in ent[pid1.ns]: # pid1 exist
          if not pid2.ns in ent: # create namespace
            ent[pid2.ns]=[]
          if not pid2.key in ent[pid2.ns]:
            ent[pid2.ns].append(pid2.key)
          return True
        if pid2.key in ent[pid2.ns]: # pid1 exist
          if not pid1.ns in ent: # create namespace
            ent[pid1.ns]=[]
          if not pid1.key in ent[pid1.ns]:
            ent[pid1.ns].append(pid1.key)
          return True
    # if they do not exist:
    new_alias = {}
    new_alias[pid1.ns]=[pid1.key]
    if pid1.ns == pid2.ns:
      new_alias[pid2.ns].append(pid2.key)
    else:
      new_alias[pid2.ns]=[pid2.key]
      
    self.alias[ptype].append(new_alias)
    return True

  def getAlias(self, node):
    " getAliases and self not used currently"
    if not node.ptype in self.alias:
      return [node.pid];
    for ent in self.alias[node.ptype]: # for each entity
      if node.pid.ns in ent: # if the namespace is registered
        if node.pid.key in ent[node.pid.ns]: # if found
          l = []
          for ns in ent:
            for key in ent[ns]:
              l.append(pId(key, ns))
          return l      
    return [node.pid]
    '''
  '''
  def get(self, node, ptype):
    "handles all relations"
    #self.querycount = querycount + 1
    #print "Counter:", self.querycount, ptype
    if   node.ptype == ptype:           return [self.exist(node)]
    elif ptype == pUpstream:            return self.upstream(node)
    elif ptype == pDownstream:          return self.downstream(node)
    elif ptype == pAnnotation:          return self.getAnnotation(node)
    elif node.ptype == pOutputPort:
      if   ptype == pDataItem:          return self.outputHasData(node)
    elif node.ptype == pDataItem:
      if   ptype == pModuleExecution:   return self.getModuleExecutionFromData(node)
      elif ptype == pParent:            return self.derivedFrom(node)
      elif ptype == pChild:             return self.derivedData(node)
      elif ptype == pAnnotation:        return self.getAnnotation(node)
      elif ptype == pOutputPort:        return self.getOutputPortFromData(node)
    elif node.ptype == pModuleExecution:
      if   ptype == pOutputPort:        return self.getOutputDataFromModuleExecution(node)
      elif ptype == pInputPort:         return self.getInputDataFromModuleExecution(node)
      elif ptype == pModuleInstance:    return self.getModuleInstanceFromModuleExecution(node)
    print 'Function', node.ptype, '-->', ptype, 'is not implemented'
  '''
  ' functions for handling dependencies in other sources '   
  def addConn(self, pid1, pid2):
    for c in self.connList:
      if c[0] == pid1 and c[1] == pid2:
        return self
    self.connList.append([pid1, pid2])
    return self
  
  def getUpConn(self, pid2):
    res = []
    for c in self.connList:
      if c[1] == pid2:
        res.append(c[0])
    return res
    
  def getDownConn(self, pid1):
    res = []
    for c in self.connList:
      if c[0] == pid1:
        res.append(c[1])
    return res
    
  def upstream(self, node, limit=0, stop=[]):
    "returns all data that contributed directly"
    if node.ptype == pModuleExecution:
      if self.execGraph == False:
        self.initExecGraph()
      graph = self.execGraph
    elif node.ptype == pDataItem:
      if self.execGraph == False:
        self.initDataGraph()
      graph = self.dataGraph
    elif node.ptype == pModuleInstance:
      if self.instGraph == False:
        self.initInstGraph()
      graph = self.instGraph
    else:
      return []
    newdataitems = []
    # process node on current source
    for [e1,e2] in graph.upstream(node.pid.key, limit, stop):
      n1 = PQObject.PQObject(node.pqf, node.ptype, e1, self.ns)
      n2 = PQObject.PQObject(node.pqf, node.ptype, e2, self.ns)
      newdataitems.append([n1, n2])

    # process new nodes having upstream in other sources
    pids = [node.pid] # add self
    for [n1, n2] in newdataitems: # add new nodes
      if not n1.pid in pids: pids.append(n1.pid)
      if not n2.pid in pids: pids.append(n2.pid)
    externalpidlist = []
    for p in pids: # check for upstreams
      if p.key not in stop:
        for ep in self.getUpConn(p): # if having upConn in other source
          if ep not in externalpidlist: # if not already found
            # add external conn to result
            n1 = PQObject.PQObject(node.pqf, node.ptype, ep.key, ep.ns)
            n2 = PQObject.PQObject(node.pqf, node.ptype, p.key, p.ns)
            newdataitems.append([n1, n2])
            # add to processed list
            externalpidlist.append(ep)
    # calc the upstream in other sources
    for p in externalpidlist:
      n = PQObject.PQObject(node.pqf, node.ptype, p.key, p.ns)
      for [n1,n2] in n.upstream(limit, stop):
        newdataitems.append([n1, n2])
    return newdataitems

  def downstream(self, node, limit=0, stop=[]):
    "returns all data that contributed directly"
    if node.ptype == pModuleExecution:
      if self.execGraph == False:
        self.initExecGraph()
      graph = self.execGraph
    elif node.ptype == pDataItem:
      if self.execGraph == False:
        self.initDataGraph()
      graph = self.dataGraph
    elif node.ptype == pModuleInstance:
      if self.instGraph == False:
        self.initInstGraph()
      graph = self.instGraph
    else:
      return []
    newdataitems = []

    # process node on current source
    for [e1,e2] in graph.downstream(node.pid.key, limit, stop):
      n1 = PQObject.PQObject(node.pqf, node.ptype, e1, self.ns)
      n2 = PQObject.PQObject(node.pqf, node.ptype, e2, self.ns)
      newdataitems.append([n1, n2])
    # process new nodes having downstream in other sources
    pids = [node.pid] # add self
    for [n1, n2] in newdataitems: # add new nodes
      if not n1.pid in pids: pids.append(n1.pid)
      if not n2.pid in pids: pids.append(n2.pid)
    externalpidlist = []
    for p in pids: # check for downstreams
      if p.key not in stop:
        for ep in self.getDownConn(p): # if having downConn in other source
          if ep not in externalpidlist: # if not already found
            # add external conn to result
            n1 = PQObject.PQObject(node.pqf, node.ptype, p.key, p.ns)
            n2 = PQObject.PQObject(node.pqf, node.ptype, ep.key, ep.ns)
            newdataitems.append([n1, n2])
            # add to processed list
            externalpidlist.append(ep)
    # calc the downstream in other sources
    for p in externalpidlist:
      n = PQObject.PQObject(node.pqf, node.ptype, p.key, p.ns)
      for [n1,n2] in n.downstream(limit, stop):
        newdataitems.append([n1, n2])
    return newdataitems


