from pTypes import *
import XMLStore, libxml2, PQObject, PGraph

class PasoaStore(XMLStore.XMLStore):
  def __init__(self, ns, path):
    " Adds Pasoa ns:s and calls XMLStore constructor "
    xmlns = dict()   
    xmlns['ps'] = "http://www.pasoa.org/schemas/version025/PStruct.xsd" # (#)
    XMLStore.XMLStore.__init__(self, ns, path, xmlns)
    self.execGraph = False

  def initExecGraph(self):
    self.execGraph = PGraph.PGraph()

    # building mappings
    rl = self.getAll(pModuleExecution)
    for r in rl:
      r.lin = r.getInDataFromExecution()
      r.lout = r.getOutDataFromExecution()

    for r1 in rl:
      for r2 in rl:
        for out in r1.lout:
          if out in r2.lin and r1.pid.ns == r2.pid.ns:
            # we have a match
            self.execGraph.add(r1.pid.key, r2.pid.key)



  #### relations implemented #### 
  def exist(self, node):
    return True
  
  def derivedFrom(self, node):
    objectIdPath = "/ps:pstruct/ps:interactionRecord/ps:sender[ps:interactionPAssertion/ps:content/ps:interactionPAssertion/ps:content/java/object/void[@property='result']//string='" + node.pid + "']/ps:relationshipPAssertion/ps:objectId"    
    objectIds = self.context.xpathEval(objectIdPath)
    newdataitems = []
    for o in objectIds:
      osource = libxml2.parseDoc(o.__str__())
      ocontext = osource.xpathNewContext()
      ocontext.xpathRegisterNs("ps", "http://www.pasoa.org/schemas/version025/PStruct.xsd")
#      q = "/objectId/interactionKey/interactionId"
#      interactionId = ocontext.xpathEval(q)[0].get_children()
      q = "/objectId/localPAssertionId"
      localPAssertionId = ocontext.xpathEval(q)[0].get_children()
      q = "/objectId/dataAccessor/xPath"
      xPath = ocontext.xpathEval(q)[0].get_children()
      pid = 'pasoa_dataLink:' + localPAssertionId.__str__() + '+' + xPath.__str__()
      n = node.pqf.getNode(pid, pDataItem)
      if n:      
        n.pqf.getAlias(n) # resolves alias
        newdataitems.append(n)
      #else:
      #  print 'Warning data not found: ' + pid
    return newdataitems

  def getExecutionFromOutData(self, node):
    rl = self.getAll(pModuleExecution)
    for r in rl:
      if node in r.getOutDataFromExecution():
        return r
    return []

  def getAll(self, ptype):
    newdataitems = []
    if ptype == pModuleExecution:
# removes trivial modules
      q = '//ps:relationshipPAssertion[ps:relation!="http://relation.org/equal" and ps:relation!="http://relation.org/isConfigurationData" and ps:relation!="http://relation.org/isElement1Of" and ps:relation!="http://relation.org/isElement2Of" and ps:relation!="http://relation.org/isParameter"]/ps:localPAssertionId'
#      q = '//ps:relationshipPAssertion/ps:localPAssertionId'
      dataitems = self.context.xpathEval(q)
      for d in dataitems:
        newdataitems.append(PQObject.PQObject(self.pqf, ptype, d.getContent(), self.ns))
    elif ptype == pModuleInstance:

      q = '//ps:relation'
      dataitems = self.context.xpathEval(q)
      for d in dataitems:
        newdataitems.append(PQObject.PQObject(self.pqf, ptype, d.getContent(), self.ns))
    else:
      return []
    return newdataitems

  def getAnnotation(self, node):
    newdataitems = dict()
    if node.ptype == pModuleExecution:
      q = '//ps:relationshipPAssertion[ps:localPAssertionId="' + node.pid.key + '"]/..//void[@property="date"]//long'
      for d in self.context.xpathEval(q):
        newdataitems['time']=d.getContent()
    elif node.ptype == pModuleInstance:
      pass
    return newdataitems

  def getInstanceFromExecution(self, node):
    q = "//ps:relationshipPAssertion[ps:localPAssertionId='" + node.pid.key + "']/ps:relation"
    dataitems = self.context.xpathEval(q)
    newdataitems = []
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleInstance, d.getContent(), self.ns))
    return newdataitems

  def getExecutionFromInstance(self, node):
    q = "//ps:relationshipPAssertion[ps:relation='" + node.pid.key + "']/ps:localPAssertionId"
    dataitems = self.context.xpathEval(q)
    newdataitems = []
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleExecution, d.getContent(), self.ns))
    return newdataitems

  def getModuleFromInstance(self, node):
    return [node]

  def getOutDataFromExecution(self, node):
    q = "//ps:relationshipPAssertion[ps:localPAssertionId='" + node.pid.key + "']/ps:subjectId/ps:dataAccessor/xPath"
    xpath = self.context.xpathEval(q)[0].getContent() + '//object' # replace fixes attribute bug
    q = "//ps:relationshipPAssertion[ps:localPAssertionId='" + node.pid.key + "']/ps:subjectId/ps:localPAssertionId"
    aid = self.context.xpathEval(q)[0].getContent()
    q = "/ps:pstruct/ps:interactionRecord/ps:sender/ps:interactionPAssertion/ps:content/ps:interactionPAssertion[ps:localPAssertionId='" + aid + "']/ps:content/" + xpath
    dataitems = self.context.xpathEval(q)
    newdataitems = []
    for d in dataitems:
      d = d.getContent().split('</string>')[0].split('<string>')[0].strip()
      newdataitems.append(PQObject.PQObject(self.pqf, pDataItem, d, self.ns))
    return newdataitems
    
  def getInDataFromExecution(self, node):
    newdataitems = []
    q = "//ps:relationshipPAssertion[ps:localPAssertionId='" + node.pid.key + "']/ps:objectId"
    objects = self.context.xpathEval(q)
    for o in objects:
      aid = o.serialize().split('</localPAssertionId>')[0].split('<localPAssertionId>')[1].strip()
      xpath = o.serialize().split('</xPath>')[0].split('<xPath xmlns="">')[1].strip().replace( '/object', '//object' ) + "//object"
      q = "/ps:pstruct/ps:interactionRecord/ps:receiver/ps:interactionPAssertion/ps:content/ps:interactionPAssertion[ps:localPAssertionId='" + aid + "']/ps:content/" + xpath
      dataitems = self.context.xpathEval(q)
      for d in dataitems:
        d = d.getContent().split('</string>')[0].split('<string>')[0].strip()
        newdataitems.append(PQObject.PQObject(self.pqf, pDataItem, d, self.ns))
    return newdataitems

