from pTypes import *
import RDFStore, PQObject, PGraph, time, datetime

class TavernaRDFStore(RDFStore.RDFStore):
  def __init__(self, ns, path):
    " Adds Taverna ns:s and calls RDFStore constructor "
    xmlns = dict()
    xmlns['j.0'] = "http://www.mygrid.org.uk/provenance#"
    xmlns['rdf'] = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    RDFStore.RDFStore.__init__(self, ns, path, xmlns)
    self.dataGraph = self.execGraph = False
  def initExecGraph(self):
    # create execGraph and populate
    q = '''
    SELECT ?e1 ?e2
    FROM <''' + self.path + '''>
    WHERE 
      { ?e1 <http://www.mygrid.org.uk/provenance#processInput> ?nextdata .
        ?e1 <http://www.mygrid.org.uk/provenance#processOutput> ?data .
        ?e2 <http://www.mygrid.org.uk/provenance#processInput> ?data .
        ?e2 <http://www.mygrid.org.uk/provenance#processOutput> ?prevdata . }
    ''' 
    dataitems = self.context.query(q)
    self.execGraph = PGraph.PGraph()
    for (e1,e2) in dataitems:
      self.execGraph.add(e1, e2)
      
  def initDataGraph(self):
    # create dataGraph and populate
    q = '''
    SELECT ?d1 ?d2
    FROM <''' + self.path + '''>
    WHERE 
      { ?d2 <http://www.mygrid.org.uk/provenance#dataDerivedFrom> ?d1}
    ''' 
    dataitems = self.context.query(q)
    self.dataGraph = PGraph.PGraph()
    for (e1,e2) in dataitems:
      self.dataGraph.add(e1, e2)


  #### relations implemented #### 
  def exist(self, node):
    """ determines if the node exist in the source 
        e.g. if node appears as subject in a triple"""
    q = '''
    SELECT ?p
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> ?p ?o }
    '''
    r = self.context.query(q)
    for (e,) in r:
      return True
    return False
  
  def derivedFrom(self, node):
    " returns all data that contributed directly "
    q = '''
    SELECT ?ddata
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#dataDerivedFrom> ?ddata}
    ''' 
    return self.processQueryAsList(q, pDataItem)

  def getAll(self, ptype):
    " returns all data that contributed directly "
    if ptype == pWorkflowExecution:
      q = ' ?o <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.mygrid.org.uk/provenance#WorkflowRun> ' 
    elif ptype == pModuleExecution:
      q = ''' ?o <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.mygrid.org.uk/provenance#ProcessRun> .
            ?o <http://www.mygrid.org.uk/provenance#processInput> ?nextdata '''
    elif ptype == pDataItem:
      q = ' ?o <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.mygrid.org.uk/provenance#DataObject> ' 
    else:
      return []
    q = ''' SELECT ?o
    FROM <''' + self.path + '''>
    WHERE { ''' + q + ''' } ''' 
    return self.processQueryAsList(q, ptype)

  def getDataFromOutPort(self, node):
    "  "
    q = '''
    SELECT ?output
    FROM <''' + self.path + '''>
    WHERE 
      { ?output <http://www.mygrid.org.uk/provenance#outputDataHasName> <''' + node.pid.key + '''> }
    ''' 
    return self.processQueryAsList(q, pDataItem)

  def getOutPortFromData(self, node):
    "  "
    q = '''
    SELECT ?out
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#outputDataHasName> ?out }
    ''' 
    return self.processQueryAsList(q, pDataItem)

  def getExecutionFromInData(self, node):
    "  "
    q = '''
    SELECT ?exec
    FROM <''' + self.path + '''>
    WHERE 
      { ?exec <http://www.mygrid.org.uk/provenance#processInput> <''' + node.pid.key + '''> }
    ''' 
    return self.processQueryAsList(q, pModuleExecution)

  def getExecutionFromOutData(self, node):
    "  "
    q = '''
    SELECT ?exec
    FROM <''' + self.path + '''>
    WHERE 
      { ?exec <http://www.mygrid.org.uk/provenance#processOutput> <''' + node.pid.key + '''> }
    ''' 
    return self.processQueryAsList(q, pModuleExecution)

  def getInDataFromExecution(self, node):
    "  "
    q = '''
    SELECT ?data
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#processInput> ?data }
    ''' 
    return self.processQueryAsList(q, pDataItem)

  def getOutDataFromExecution(self, node):
    "  "
    q = '''
    SELECT ?data
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#processOutput> ?data }
    ''' 
    return self.processQueryAsList(q, pDataItem)

  def getRunFromExecution(self, node):
    "  "
    q = '''
    SELECT ?data
    FROM <''' + self.path + '''>
    WHERE 
      { ?data <http://www.mygrid.org.uk/provenance#executedProcessRun> <''' + node.pid.key + '''> }
    ''' 
    return self.processQueryAsList(q, pWorkflowExecution)

  def getExecutionFromRun(self, node):
    "  "
    q = '''
    SELECT ?data
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#executedProcessRun> ?data }
     ''' 
    return self.processQueryAsList(q, pModuleExecution)

  def getRunFromWorkflow(self, node):
    "  "
    q = '''
    SELECT ?data
    FROM <''' + self.path + '''>
    WHERE 
      { ?data <http://www.mygrid.org.uk/provenance#runsWorkflow> <''' + node.pid.key + '''> }
    ''' 
    return self.processQueryAsList(q, pWorkflowExecution)

  def getWorkflowFromRun(self, node):
    "  "
    q = '''
    SELECT ?data
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#runsWorkflow> ?data }
     ''' 
    return self.processQueryAsList(q, pWorkflow)

  def getAnnotation(self, node):
    " returns all annotations "
    if node.ptype == pModuleExecution:
      q = '''
        } UNION {
        <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#processInput> ?data .
        ?e1 <http://www.mygrid.org.uk/provenance#processOutput> ?data .
        OPTIONAL { ?e1 <http://www.mygrid.org.uk/provenance#processInput> ?nextdata } .
        FILTER (!bound(?nextdata)) .
        ?e1 <http://www.mygrid.org.uk/provenance#runsProcess> ?key .
        ?key <http://www.mygrid.org.uk/provenance#hasProperty> ?prop .
        ?prop <http://www.mygrid.org.uk/provenance#key> ?type .
        FILTER(?type = 'String Value') .
        ?prop <http://www.mygrid.org.uk/provenance#value> ?value .
      ''' 
    elif node.ptype == pWorkflowExecution:
      q = '''
        } UNION {
        <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#launchedBy> ?user .
        ?user <http://www.mygrid.org.uk/provenance#belongsTo> ?value .
        ?user ?key ?value .
       ''' 
    elif node.ptype == pDataItem:
      q = ''' ''' 
    else:
      q = ''' ''' 
    q = ''' SELECT ?key ?value FROM 
        <''' + self.path + '''> WHERE {{ 
        <''' + node.pid.key + '''> ?key ?value .
        ''' + q + '''
        }} ''' 
    return self.processQueryAsDict(q)

  def getInstanceFromExecution(self, node):
    "  "
    q = '''
    SELECT ?mi
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#runsProcess> ?mi }
    ''' 
    return self.processQueryAsList(q, pModuleInstance)

  def getModuleFromInstance(self, node):
    "  "
    q = '''
    SELECT ?mo
    FROM <''' + self.path + '''>
    WHERE 
      { <''' + node.pid.key + '''> <http://www.mygrid.org.uk/provenance#className> ?mo }
    ''' 
    return self.processQueryAsList(q, pModule)

  def getChildDataItem(self, node):
    "returns all data that contributed directly"
    q = '''
    SELECT ?data
    FROM <''' + self.path + '''>
    WHERE 
      { ?data <http://www.mygrid.org.uk/provenance#dataDerivedFrom> <''' + node.pid.key + '''> }
    ''' 
    return self.processQueryAsList(q, pDataItem)

  def dayOfWeek(self, string, day):
    weekdayMap = {0:'Monday', 1:'Tuesday', 2:'Wednesday', 3:'Thursday', 4:'Friday', 5:'Saturday', 6:'Sunday'}
    return day.lower()[:2] == weekdayMap[datetime.datetime(*time.strptime(string[:20], '"%Y-%m-%dT%H:%M:%S')[0:6]).weekday()].lower()[:2]

