from pTypes import *
import XMLStore, libxml2, PQObject, PGraph, time, datetime

class VisTrailsStore(XMLStore.XMLStore):
  def __init__(self, ns, path):
    " Adds VisTrails ns:s and calls XMLStore constructor "
    xmlns = dict()
    XMLStore.XMLStore.__init__(self, ns, path, xmlns)
    # the log is hardcoded :-(
    logpath = '../VisTrails/pc_log.xml'
    self.logsource = libxml2.parseDoc(file(logpath).read())
    self.logcontext = self.logsource.xpathNewContext()
    self.instGraph = False
    self.execGraph = False

  def initInstGraph(self):
    self.instGraph = PGraph.PGraph()
    # get all modules
    q = '//module/@id'
    instances = self.context.xpathEval(q)
    # for each module, find all children
    for i1 in instances:
      q = '//connection[port[@type="source"]/@moduleId="' + i1.getContent() + '"]/port[@type="destination"]/@moduleId'
      instances2 = self.context.xpathEval(q)
      for i2 in instances2:
        self.instGraph.add(i1.getContent(), i2.getContent())

  def initExecGraph(self):
    self.execGraph = PGraph.PGraph()
    # get all modules
    q = '//module/@id'
    instances = self.context.xpathEval(q)
    # for each module, find all children
    for i1 in instances:
      i1 = i1.getContent()
      q = '//connection[port[@type="source"]/@moduleId="' + i1 + '"]/port[@type="destination"]/@moduleId'
      e1 = self.pqf.getNode(pModuleInstance, i1, self.ns).getExecutionFromInstance()[0].pid.key
      instances2 = self.context.xpathEval(q)
      for i2 in instances2:
        e2 = self.pqf.getNode(pModuleInstance, i2.getContent(), self.ns).getExecutionFromInstance()[0].pid.key
        self.execGraph.add(e1, e2)

  #### relations implemented #### 
  def exist(self, node):
    " determines if the node exist in the source "
    q = 'local-name(//*[@rdf:about="' + node.pid.key + '"])'
    return True #self.context.xpathEval(q) != []
  
  def getInstanceFromModule(self, node):
    newdataitems = []
    q = '//module[@name="' + node.pid.key + '"]/@id'
    dataitems = self.context.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleInstance, d.getContent(), self.ns))
    return newdataitems
    
  def getInstanceFromWorkflow(self, node):
    newdataitems = []
    q = '//workflow[@id="' + node.pid.key + '"]/module/@id'
    dataitems = self.context.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleInstance, d.getContent(), self.ns))
    return newdataitems

  def getInstanceFromExecution(self, node):
    newdataitems = []
    q = '//exec[@id="' + node.pid.key + '"]/@moduleId'
    dataitems = self.logcontext.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleInstance, d.getContent(), self.ns))
    return newdataitems

  def getRunFromExecution(self, node):
    newdataitems = []
    q = '//wfExec[exec/@id="' + node.pid.key + '"]/@id'
    dataitems = self.logcontext.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pWorkflowExecution, d.getContent(), self.ns))
    return newdataitems

  def getWorkflowFromRun(self, node):
    newdataitems = []
    q = '//wfExec[@id="' + node.pid.key + '"]/@wfVersion'
    dataitems = self.logcontext.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pWorkflow, d.getContent(), self.ns))
    return newdataitems

  def getExecutionFromInstance(self, node):
    newdataitems = []
    for wf in self.getAll(pWorkflow):
      q = '//wfExec[@wfVersion="' + wf.pid.key + '"]//exec[@moduleId="' + node.pid.key + '"]/@id'
      dataitems = self.logcontext.xpathEval(q)
      for d in dataitems:
        newdataitems.append(PQObject.PQObject(self.pqf, pModuleExecution, d.getContent(), self.ns))
    return newdataitems

  def getModuleFromInstance(self, node):
    newdataitems = []
    q = '//module[@id="' + node.pid.key + '"]/@name'
    dataitems = self.context.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleInstance, d.getContent(), self.ns))
    return newdataitems

  def getParentInstance(self, node):
    newdataitems = []
    q = '//connection[port[@type="source"]/@moduleId="' + node.pid.key + '"]/port[@type="destination"]/@moduleId'
    dataitems = self.context.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleInstance, d.getContent(), self.ns))
    return newdataitems

  def getChildInstance(self, node):
    newdataitems = []
    q = '//connection[port[@type="destination"]/@moduleId="' + node.pid.key + '"]/port[@type="source"]/@moduleId'
    dataitems = self.context.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleInstance, d.getContent(), self.ns))
    return newdataitems
    
  def getParentExecution(self, node):
    i1 = node.getInstanceFromExecution()
    newdataitems = []
    q = '//module[@id="' + node.pid.key + '"]/@name'
    dataitems = self.context.xpathEval(q)
    for d in dataitems:
      newdataitems.append(PQObject.PQObject(self.pqf, pModuleInstance, d.getContent(), self.ns))
    return newdataitems

  def getAnnotation(self, node):
    newdataitems = dict()
    if node.ptype == pModuleExecution:
      q = '//exec[@id="' + node.pid.key + '"]/annotation'
      for d in self.logcontext.xpathEval(q):
        newdataitems[d.hasProp('key').getContent()]=d.hasProp('value').getContent()
      q = '//exec[@id="' + node.pid.key + '"]/@tsStart'
      for d in self.logcontext.xpathEval(q):
        newdataitems['tsStart']=d.getContent()
      q = '//exec[@id="' + node.pid.key + '"]/@tsEnd'
      for d in self.logcontext.xpathEval(q):
        newdataitems['tsEnd']=d.getContent()   

    if node.ptype == pModuleInstance:
      q = '//module[@id="' + node.pid.key + '"]/function/parameter'
      for d in self.context.xpathEval(q):
        newdataitems[d.get_parent().hasProp('name').getContent()]=d.hasProp('val').getContent()
      q = '//module[@id="' + node.pid.key + '"]/annotation'
      for d in self.context.xpathEval(q):
        newdataitems[d.hasProp('key').getContent()]=d.hasProp('value').getContent()
    return newdataitems

  def getAll(self, ptype):
    newdataitems = []
    if ptype == pWorkflow:
      q = '//workflow/@id'
      dataitems = self.context.xpathEval(q)
      for d in dataitems:
        newdataitems.append(PQObject.PQObject(self.pqf, ptype, d.getContent(), self.ns))
    elif ptype == pWorkflowExecution:
      for wf in self.getAll(pWorkflow):
        q = '//wfExec[@wfVersion="' + wf.pid.key + '"]/@id'
        dataitems = self.logcontext.xpathEval(q)
        for d in dataitems:
          newdataitems.append(PQObject.PQObject(self.pqf, ptype, d.getContent(), self.ns))
    elif ptype == pModuleExecution:
      for wf in self.getAll(pWorkflow):
        q = '//wfExec[@wfVersion="' + wf.pid.key + '"]/exec/@id'
        dataitems = self.logcontext.xpathEval(q)
        for d in dataitems:
          newdataitems.append(PQObject.PQObject(self.pqf, ptype, d.getContent(), self.ns))
    elif ptype == pModuleInstance:
      q = '//module/@id'
      dataitems = self.context.xpathEval(q)
      for d in dataitems:
        newdataitems.append(PQObject.PQObject(self.pqf, ptype, d.getContent(), self.ns))
    elif ptype == pModule:
      q = '//module/@name'
      dataitems = self.context.xpathEval(q)
      for d in dataitems:
        newdataitems.append(PQObject.PQObject(self.pqf, ptype, d.getContent(), self.ns))
    else:
      return []
    return newdataitems

  def dayOfWeek(self, string, day):
    weekdayMap = {0:'Monday', 1:'Tuesday', 2:'Wednesday', 3:'Thursday', 4:'Friday', 5:'Saturday', 6:'Sunday'}
    return day.lower()[:2] == weekdayMap[datetime.datetime(*time.strptime(string, '%Y-%m-%dT%H:%M:%S')[0:6]).weekday()].lower()[:2]

